@@ -1,6 +1,6 @@
 package CPAN::Faker;
 BEGIN {
-  $CPAN::Faker::VERSION = '0.005';
+  $CPAN::Faker::VERSION = '0.006';
 }
 use 5.008;
 use Moose;
@@ -10,6 +10,7 @@ use CPAN::Checksums ();
 use Compress::Zlib ();
 use Cwd ();
 use Data::Section -setup;
+use File::Find ();
 use File::Next ();
 use File::Path ();
 use File::Spec ();
@@ -19,7 +20,7 @@ use Text::Template;
 
 
 has dest   => (is => 'ro', isa => 'Str', required => 1);
-has source => (is => 'ro', isa => 'Str', required => 0);
+has source => (is => 'ro', isa => 'Str', required => 1);
 
 has dist_class => (
   is  => 'ro',
@@ -59,29 +60,29 @@ BEGIN {
   }
 }
 
-sub _validate_dir {
-  my ($self, $type) = @_;
-
-  my $dir = $self->$type;
-  Carp::croak "$type directory not supplied"       unless $dir;
-  Carp::croak "$type directory does not exist"     unless -e $dir;
-  Carp::croak "$type directory is not a directory" unless -d $dir;
-  Carp::croak "$type directory is not writeable"   unless -w $dir;
-}
-
-sub BUILD {
-  my ($self) = @_;
-
-  $self->_validate_dir('dest');
-}
-
 sub __dor { defined $_[0] ? $_[0] : $_[1] }
 
 
 sub make_cpan {
-  my ($self, $arg) = @_;
+  my ($self) = @_;
 
-  $self->_validate_dir('source');
+  for ($self->source) {
+    Carp::croak "source directory does not exist"     unless -e;
+    Carp::croak "source directory is not a directory" unless -d;
+    Carp::croak "source directory is not writeable"   unless -w;
+  }
+
+  for ($self->dest) {
+    if (-e) {
+      Carp::croak "destination directory is not a directory" unless -d;
+
+      opendir my $dir, $self->dest;
+      my @files = grep { $_ ne '.' and $_ ne '..' } readdir $dir;
+      Carp::croak "destination directory is not empty" if @files;
+    } else {
+      Carp::croak "couldn't create destination: $!" unless mkdir;
+    }
+  }
 
   my $iter = File::Next::files($self->source);
 
@@ -235,7 +236,9 @@ sub write_package_index {
   my $index = $self->_pkg_index;
 
   my @lines;
-  while (my ($name, $info) = each %$index) {
+  for my $name (sort keys %$index) {
+    my $info = $index->{ $name };
+
     push @lines, sprintf "%-34s %5s  %s\n",
       $name,
       __dor($info->{version}, 'undef'),
@@ -341,7 +344,7 @@ CPAN::Faker - build a bogus CPAN instance for testing
 
 =head1 VERSION
 
-version 0.005
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -374,8 +377,7 @@ should be in your fake CPAN.
 This create the new CPAN::Faker.  All arguments may be accessed later by
 methods of the same name.  Valid arguments are:
 
-  source - the directory in which to find source files; optional, required by
-           make_cpan
+  source - the directory in which to find source files
   dest   - the directory in which to construct the CPAN instance; required
   url    - the base URL for the CPAN; a file:// URL is generated by default
 
